@@ -11,9 +11,9 @@ version 3 of the License, or (at your option) any later version.
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-Lesser General Public License for more details.
+General Public License for more details.
 
-You should have received a copy of the GNU Lesser General Public
+You should have received a copy of the GNU General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 """
@@ -54,19 +54,16 @@ class PyMataCommandHandler(threading.Thread):
     REPORT_VERSION = 0xF9  # report protocol version
 
     # user defined SYSEX commands
-    # from this client
     ENCODER_CONFIG = 0x20  # create and enable encoder object
     TONE_PLAY = 0x5F  # play a tone at a specified frequency and duration
     SONAR_CONFIG = 0x60  # configure pins to control a Ping type sonar distance device
 
-    # messages from firmata
     ENCODER_DATA = 0x21  # current encoder position data
     SONAR_DATA = 0x61  # distance data returned
 
-    # standard Firmata sysex commands
-
     SERVO_CONFIG = 0x70  # set servo pin and max and min angles
     STRING_DATA = 0x71  # a string message with 14-bits per char
+    STEPPER_DATA = 0x72  # Stepper motor command
     I2C_REQUEST = 0x76  # send an I2C read/write request
     I2C_REPLY = 0x77    # a reply to an I2C read request
     I2C_CONFIG = 0x78   # config I2C settings such as delay times and power pins
@@ -85,25 +82,6 @@ class PyMataCommandHandler(threading.Thread):
     SYSEX_NON_REALTIME = 0x7E  # MIDI Reserved for non-realtime messages
     SYSEX_REALTIME = 0x7F  # MIDI Reserved for realtime messages
 
-    # pin modes
-    INPUT = 0x00  # pin set as input
-    OUTPUT = 0x01  # pin set as output
-    ANALOG = 0x02  # analog pin in analogInput mode
-    PWM = 0x03  # digital pin in PWM output mode
-    SERVO = 0x04  # digital pin in Servo output mode
-    I2C = 0x06  # pin included in I2C setup
-    ONEWIRE = 0x07  # possible future feature
-    STEPPER = 0x08  # possible future feature
-    TONE = 0x09  # Any pin in TONE mode
-    ENCODER = 0x0a
-    SONAR = 0x0b  # Any pin in SONAR mode
-    IGNORE = 0x7f
-
-    # the following pin modes are not part of or defined by Firmata
-    # but used by PyFirmata
-
-    DIGITAL = 0x20
-
     # The response tables hold response information for all pins
     # Each table is a table of entries for each pin, which consists of the pin mode, and its last value from firmata
 
@@ -195,14 +173,18 @@ class PyMataCommandHandler(threading.Thread):
     # the active_sonar_map maps the sonar trigger pin number (the key) to the current data value returned
     active_sonar_map = {}
 
-    def __init__(self, transport, command_deque, data_lock):
+    # the stepper library version number.
+    stepper_library_version = 0
+
+    def __init__(self, pymata):
         """
         constructor for CommandHandler class
-        @param transport: A reference to the communications port designator.
-        @param command_deque:  A reference to a command deque.
-        @param data_lock: A reference to a thread lock.
+        @param pymata: A reference to the pymata instance.
         """
 
+        # reference pointer to pymata
+        self.pymata = pymata
+
         # this list contains the results of the last pin query
         self.last_pin_query_results = []
 
@@ -212,16 +194,20 @@ class PyMataCommandHandler(threading.Thread):
         # this stores the results of an analog mapping query
         self.analog_mapping_query_results = []
 
-        self.data_lock = data_lock
-        self.transport = transport
-        self.command_deque = command_deque
-
         self.total_pins_discovered = 0
 
         self.number_of_analog_pins_discovered = 0
 
         threading.Thread.__init__(self)
         self.daemon = True
+        
+        self.stop_event = threading.Event()
+
+    def stop(self):
+        self.stop_event.set()
+
+    def is_stopped(self):
+        return self.stop_event.is_set()
 
     def auto_discover_board(self):
         """
@@ -246,7 +232,7 @@ class PyMataCommandHandler(threading.Thread):
         for pin in self.analog_mapping_query_results:
             self.total_pins_discovered += 1
             # non analog pins will be marked as IGNORE
-            if pin != self.IGNORE:
+            if pin != self.pymata.IGNORE:
                 self.number_of_analog_pins_discovered += 1
 
         print 'Total Number of Pins Detected = %d' % self.total_pins_discovered
@@ -255,11 +241,11 @@ class PyMataCommandHandler(threading.Thread):
         # response table initialization
         # for each pin set the mode to input and the last read data value to zero
         for pin in range(0, self.total_pins_discovered):
-            response_entry = [self.INPUT, 0]
+            response_entry = [self.pymata.INPUT, 0]
             self.digital_response_table.append(response_entry)
 
         for pin in range(0, self.number_of_analog_pins_discovered):
-            response_entry = [self.INPUT, 0]
+            response_entry = [self.pymata.INPUT, 0]
             self.analog_response_table.append(response_entry)
 
         # set up latching tables
@@ -292,9 +278,8 @@ class PyMataCommandHandler(threading.Thread):
         @param threshold_type: ANALOG_LATCH_GT | ANALOG_LATCH_LT  | ANALOG_LATCH_GTE | ANALOG_LATCH_LTE
         @param threshold_value: numerical value
         """
-        self.data_lock.acquire(True)
-        self.analog_latch_table[pin] = [self.LATCH_ARMED, threshold_type, threshold_value, 0, 0]
-        self.data_lock.release()
+        with self.pymata.data_lock:
+            self.analog_latch_table[pin] = [self.LATCH_ARMED, threshold_type, threshold_value, 0, 0]
 
     def set_digital_latch(self, pin, threshold_type):
         """
@@ -302,9 +287,8 @@ class PyMataCommandHandler(threading.Thread):
         @param pin: digital pin number
         @param threshold_type: DIGITAL_LATCH_HIGH | DIGITAL_LATCH_LOW
         """
-        self.data_lock.acquire(True)
-        self.digital_latch_table[pin] = [self.LATCH_ARMED, threshold_type, 0, 0]
-        self.data_lock.release()
+        with self.pymata.data_lock:
+            self.digital_latch_table[pin] = [self.LATCH_ARMED, threshold_type, 0, 0]
 
     def get_analog_latch_data(self, pin):
         """
@@ -314,16 +298,15 @@ class PyMataCommandHandler(threading.Thread):
         @param pin:  pin number
         @return: [latch_state, latched_data, and time_stamp]
         """
-        self.data_lock.acquire(True)
-        pin_data = self.analog_latch_table[pin]
-        current_latch_data = [pin,
-                              pin_data[self.LATCH_STATE],
-                              pin_data[self.ANALOG_LATCHED_DATA],
-                              pin_data[self.ANALOG_TIME_STAMP]]
-        # if this is latched data, clear the latch table entry for this pin
-        if pin_data[self.LATCH_STATE] == self.LATCH_LATCHED:
-            self.analog_latch_table[pin] = [0, 0, 0, 0, 0]
-        self.data_lock.release()
+        with self.pymata.data_lock:
+            pin_data = self.analog_latch_table[pin]
+            current_latch_data = [pin,
+                                  pin_data[self.LATCH_STATE],
+                                  pin_data[self.ANALOG_LATCHED_DATA],
+                                  pin_data[self.ANALOG_TIME_STAMP]]
+            # if this is latched data, clear the latch table entry for this pin
+            if pin_data[self.LATCH_STATE] == self.LATCH_LATCHED:
+                self.analog_latch_table[pin] = [0, 0, 0, 0, 0]
         return current_latch_data
 
     def get_digital_latch_data(self, pin):
@@ -334,15 +317,14 @@ class PyMataCommandHandler(threading.Thread):
         @param pin:  pin number
         @return: [latch_state, latched_data, and time_stamp]
         """
-        self.data_lock.acquire(True)
-        pin_data = self.digital_latch_table[pin]
-        current_latch_data = [pin,
-                              pin_data[self.LATCH_STATE],
-                              pin_data[self.DIGITAL_LATCHED_DATA],
-                              pin_data[self.DIGITAL_TIME_STAMP]]
-        if pin_data[self.LATCH_STATE] == self.LATCH_LATCHED:
-            self.digital_latch_table[pin] = [0, 0, 0, 0]
-        self.data_lock.release()
+        with self.pymata.data_lock:
+            pin_data = self.digital_latch_table[pin]
+            current_latch_data = [pin,
+                                  pin_data[self.LATCH_STATE],
+                                  pin_data[self.DIGITAL_LATCHED_DATA],
+                                  pin_data[self.DIGITAL_TIME_STAMP]]
+            if pin_data[self.LATCH_STATE] == self.LATCH_LATCHED:
+                self.digital_latch_table[pin] = [0, 0, 0, 0]
         return current_latch_data
 
     def report_firmware(self, data):
@@ -380,63 +362,61 @@ class PyMataCommandHandler(threading.Thread):
         @param data: Message data from Firmata
         @return: No return value.
         """
-        self.data_lock.acquire(True)
+        with self.pymata.data_lock:
+            # convert MSB and LSB into an integer
+            self.analog_response_table[data[self.RESPONSE_TABLE_MODE]][self.RESPONSE_TABLE_PIN_DATA_VALUE] \
+                = (data[self.MSB] << 7) + data[self.LSB]
 
-        # convert MSB and LSB into an integer
-        self.analog_response_table[data[self.RESPONSE_TABLE_MODE]][self.RESPONSE_TABLE_PIN_DATA_VALUE] \
-            = (data[self.MSB] << 7) + data[self.LSB]
-
-        pin = data[0]
-        pin_response_data_data = self.analog_response_table[pin]
-        value = pin_response_data_data[self.RESPONSE_TABLE_PIN_DATA_VALUE]
-        # check if data is to be latched
-        # get the analog latching table entry for this pin
-        latching_entry = self.analog_latch_table[pin]
-        if latching_entry[self.LATCH_STATE] == self.LATCH_ARMED:
-            # Has the latching criteria been met
-            if latching_entry[self.LATCHED_THRESHOLD_TYPE] == self.ANALOG_LATCH_GT:
-                if value > latching_entry[self.ANALOG_LATCH_DATA_TARGET]:
-                    updated_latch_entry = latching_entry
-                    updated_latch_entry[self.LATCH_STATE] = self.LATCH_LATCHED
-                    updated_latch_entry[self.ANALOG_LATCHED_DATA] = value
-                    # time stamp it
-                    updated_latch_entry[self.ANALOG_TIME_STAMP] = time.time()
-                    self.analog_latch_table[pin] = updated_latch_entry
+            pin = data[0]
+            pin_response_data_data = self.analog_response_table[pin]
+            value = pin_response_data_data[self.RESPONSE_TABLE_PIN_DATA_VALUE]
+            # check if data is to be latched
+            # get the analog latching table entry for this pin
+            latching_entry = self.analog_latch_table[pin]
+            if latching_entry[self.LATCH_STATE] == self.LATCH_ARMED:
+                # Has the latching criteria been met
+                if latching_entry[self.LATCHED_THRESHOLD_TYPE] == self.ANALOG_LATCH_GT:
+                    if value > latching_entry[self.ANALOG_LATCH_DATA_TARGET]:
+                        updated_latch_entry = latching_entry
+                        updated_latch_entry[self.LATCH_STATE] = self.LATCH_LATCHED
+                        updated_latch_entry[self.ANALOG_LATCHED_DATA] = value
+                        # time stamp it
+                        updated_latch_entry[self.ANALOG_TIME_STAMP] = time.time()
+                        self.analog_latch_table[pin] = updated_latch_entry
+                    else:
+                        pass  # haven't hit target
+                elif latching_entry[self.LATCHED_THRESHOLD_TYPE] == self.ANALOG_LATCH_GTE:
+                    if value >= latching_entry[self.ANALOG_LATCH_DATA_TARGET]:
+                        updated_latch_entry = latching_entry
+                        updated_latch_entry[self.LATCH_STATE] = self.LATCH_LATCHED
+                        updated_latch_entry[self.ANALOG_LATCHED_DATA] = value
+                        # time stamp it
+                        updated_latch_entry[self.ANALOG_TIME_STAMP] = time.time()
+                        self.analog_latch_table[pin] = updated_latch_entry
+                    else:
+                        pass  # haven't hit target:
+                elif latching_entry[self.LATCHED_THRESHOLD_TYPE] == self.ANALOG_LATCH_LT:
+                    if value < latching_entry[self.ANALOG_LATCH_DATA_TARGET]:
+                        updated_latch_entry = latching_entry
+                        updated_latch_entry[self.LATCH_STATE] = self.LATCH_LATCHED
+                        updated_latch_entry[self.ANALOG_LATCHED_DATA] = value
+                        # time stamp it
+                        updated_latch_entry[self.ANALOG_TIME_STAMP] = time.time()
+                        self.analog_latch_table[pin] = updated_latch_entry
+                    else:
+                        pass  # haven't hit target:
+                elif latching_entry[self.LATCHED_THRESHOLD_TYPE] == self.ANALOG_LATCH_LTE:
+                    if value <= latching_entry[self.ANALOG_LATCH_DATA_TARGET]:
+                        updated_latch_entry = latching_entry
+                        updated_latch_entry[self.LATCH_STATE] = self.LATCH_LATCHED
+                        updated_latch_entry[self.ANALOG_LATCHED_DATA] = value
+                        # time stamp it
+                        updated_latch_entry[self.ANALOG_TIME_STAMP] = time.time()
+                        self.analog_latch_table[pin] = updated_latch_entry
+                    else:
+                        pass  # haven't hit target:
                 else:
-                    pass  # haven't hit target
-            elif latching_entry[self.LATCHED_THRESHOLD_TYPE] == self.ANALOG_LATCH_GTE:
-                if value >= latching_entry[self.ANALOG_LATCH_DATA_TARGET]:
-                    updated_latch_entry = latching_entry
-                    updated_latch_entry[self.LATCH_STATE] = self.LATCH_LATCHED
-                    updated_latch_entry[self.ANALOG_LATCHED_DATA] = value
-                    # time stamp it
-                    updated_latch_entry[self.ANALOG_TIME_STAMP] = time.time()
-                    self.analog_latch_table[pin] = updated_latch_entry
-                else:
-                    pass  # haven't hit target:
-            elif latching_entry[self.LATCHED_THRESHOLD_TYPE] == self.ANALOG_LATCH_LT:
-                if value < latching_entry[self.ANALOG_LATCH_DATA_TARGET]:
-                    updated_latch_entry = latching_entry
-                    updated_latch_entry[self.LATCH_STATE] = self.LATCH_LATCHED
-                    updated_latch_entry[self.ANALOG_LATCHED_DATA] = value
-                    # time stamp it
-                    updated_latch_entry[self.ANALOG_TIME_STAMP] = time.time()
-                    self.analog_latch_table[pin] = updated_latch_entry
-                else:
-                    pass  # haven't hit target:
-            elif latching_entry[self.LATCHED_THRESHOLD_TYPE] == self.ANALOG_LATCH_LTE:
-                if value <= latching_entry[self.ANALOG_LATCH_DATA_TARGET]:
-                    updated_latch_entry = latching_entry
-                    updated_latch_entry[self.LATCH_STATE] = self.LATCH_LATCHED
-                    updated_latch_entry[self.ANALOG_LATCHED_DATA] = value
-                    # time stamp it
-                    updated_latch_entry[self.ANALOG_TIME_STAMP] = time.time()
-                    self.analog_latch_table[pin] = updated_latch_entry
-                else:
-                    pass  # haven't hit target:
-            else:
-                pass
-        self.data_lock.release()
+                    pass
 
     def digital_message(self, data):
         """
@@ -454,33 +434,32 @@ class PyMataCommandHandler(threading.Thread):
         pin = port * 8
         for pin in range(pin, pin + 8):
             # shift through all the bit positions and set the digital response table
-            self.data_lock.acquire(True)
-            self.digital_response_table[pin][self.RESPONSE_TABLE_PIN_DATA_VALUE] = port_data & 0x01
-            # determine if the latch data table needs to be updated for each pin
-            latching_entry = self.digital_latch_table[pin]
-            if latching_entry[self.LATCH_STATE] == self.LATCH_ARMED:
-                if latching_entry[self.LATCHED_THRESHOLD_TYPE] == self.DIGITAL_LATCH_LOW:
-                    if (port_data & 0x01) == 0:
-                        updated_latch_entry = latching_entry
-                        updated_latch_entry[self.LATCH_STATE] = self.LATCH_LATCHED
-                        updated_latch_entry[self.DIGITAL_LATCHED_DATA] = self.DIGITAL_LATCH_LOW
-                        # time stamp it
-                        updated_latch_entry[self.DIGITAL_TIME_STAMP] = time.time()
-                    else:
-                        pass
-                elif latching_entry[self.LATCHED_THRESHOLD_TYPE] == self.DIGITAL_LATCH_HIGH:
-                    if port_data & 0x01:
-                        updated_latch_entry = latching_entry
-                        updated_latch_entry[self.LATCH_STATE] = self.LATCH_LATCHED
-                        updated_latch_entry[self.DIGITAL_LATCHED_DATA] = self.DIGITAL_LATCH_HIGH
-                        # time stamp it
-                        updated_latch_entry[self.DIGITAL_TIME_STAMP] = time.time()
-                    else:
-                        pass
-            else:
-                pass
+            with self.pymata.data_lock:
+                self.digital_response_table[pin][self.RESPONSE_TABLE_PIN_DATA_VALUE] = port_data & 0x01
+                # determine if the latch data table needs to be updated for each pin
+                latching_entry = self.digital_latch_table[pin]
+                if latching_entry[self.LATCH_STATE] == self.LATCH_ARMED:
+                    if latching_entry[self.LATCHED_THRESHOLD_TYPE] == self.DIGITAL_LATCH_LOW:
+                        if (port_data & 0x01) == 0:
+                            updated_latch_entry = latching_entry
+                            updated_latch_entry[self.LATCH_STATE] = self.LATCH_LATCHED
+                            updated_latch_entry[self.DIGITAL_LATCHED_DATA] = self.DIGITAL_LATCH_LOW
+                            # time stamp it
+                            updated_latch_entry[self.DIGITAL_TIME_STAMP] = time.time()
+                        else:
+                            pass
+                    elif latching_entry[self.LATCHED_THRESHOLD_TYPE] == self.DIGITAL_LATCH_HIGH:
+                        if port_data & 0x01:
+                            updated_latch_entry = latching_entry
+                            updated_latch_entry[self.LATCH_STATE] = self.LATCH_LATCHED
+                            updated_latch_entry[self.DIGITAL_LATCHED_DATA] = self.DIGITAL_LATCH_HIGH
+                            # time stamp it
+                            updated_latch_entry[self.DIGITAL_TIME_STAMP] = time.time()
+                        else:
+                            pass
+                else:
+                    pass
 
-            self.data_lock.release()
             # get the next data bit
             port_data >>= 1
 
@@ -495,9 +474,8 @@ class PyMataCommandHandler(threading.Thread):
         # set value so that it shows positive and negative values
         if val > 8192:
             val -= 16384
-        self.data_lock.acquire(True)
-        self.digital_response_table[data[self.RESPONSE_TABLE_MODE]][self.RESPONSE_TABLE_PIN_DATA_VALUE] = val
-        self.data_lock.release()
+        with self.pymata.data_lock:
+            self.digital_response_table[data[self.RESPONSE_TABLE_MODE]][self.RESPONSE_TABLE_PIN_DATA_VALUE] = val
 
     def sonar_data(self, data):
         """
@@ -508,20 +486,18 @@ class PyMataCommandHandler(threading.Thread):
         """
         val = int((data[self.MSB] << 7) + data[self.LSB])
         pin_number = data[0]
-        self.data_lock.acquire(True)
-        self.active_sonar_map[pin_number] = val
-        # also write it into the digital response table
-        self.digital_response_table[data[self.RESPONSE_TABLE_MODE]][self.RESPONSE_TABLE_PIN_DATA_VALUE] = val
-        self.data_lock.release()
+        with self.pymata.data_lock:
+            self.active_sonar_map[pin_number] = val
+            # also write it into the digital response table
+            self.digital_response_table[data[self.RESPONSE_TABLE_MODE]][self.RESPONSE_TABLE_PIN_DATA_VALUE] = val
 
     def get_analog_response_table(self):
         """
         This method returns the entire analog response table to the caller
         @return: The analog response table.
         """
-        self.data_lock.acquire(True)
-        data = self.analog_response_table
-        self.data_lock.release()
+        with self.pymata.data_lock:
+            data = self.analog_response_table
         return data
 
     def get_digital_response_table(self):
@@ -529,9 +505,8 @@ class PyMataCommandHandler(threading.Thread):
         This method returns the entire digital response table to the caller
         @return: The digital response table.
         """
-        self.data_lock.acquire(True)
-        data = self.digital_response_table
-        self.data_lock.release()
+        with self.pymata.data_lock:
+            data = self.digital_response_table
         return data
 
     def send_sysex(self, sysex_command, sysex_data=None):
@@ -554,7 +529,7 @@ class PyMataCommandHandler(threading.Thread):
         sysex_message += chr(self.END_SYSEX)
 
         for data in sysex_message:
-            self.transport.write(data)
+            self.pymata.transport.write(data)
 
     def send_command(self, command):
         """
@@ -567,7 +542,7 @@ class PyMataCommandHandler(threading.Thread):
             send_message += chr(i)
 
         for data in send_message:
-            self.transport.write(data)
+            self.pymata.transport.write(data)
 
     def system_reset(self):
         """
@@ -576,29 +551,26 @@ class PyMataCommandHandler(threading.Thread):
         @return: No return value
         """
         data = chr(self.SYSTEM_RESET)
-        self.transport.write(data)
+        self.pymata.transport.write(data)
 
         # response table re-initialization
         # for each pin set the mode to input and the last read data value to zero
-        self.data_lock.acquire(True)
+        with self.pymata.data_lock:
+            # remove all old entries from existing tables
+            for _ in range(len(self.digital_response_table)):
+                self.digital_response_table.pop()
 
-        # remove all old entries from existing tables
-        for _ in range(len(self.digital_response_table)):
-            self.digital_response_table.pop()
+            for _ in range(len(self.analog_response_table)):
+                self.analog_response_table.pop()
 
-        for _ in range(len(self.analog_response_table)):
-            self.analog_response_table.pop()
+            # reinitialize tables
+            for pin in range(0, self.total_pins_discovered):
+                response_entry = [self.pymata.INPUT, 0]
+                self.digital_response_table.append(response_entry)
 
-        # reinitialize tables
-        for pin in range(0, self.total_pins_discovered):
-            response_entry = [self.INPUT, 0]
-            self.digital_response_table.append(response_entry)
-
-        for pin in range(0, self.number_of_analog_pins_discovered):
-            response_entry = [self.INPUT, 0]
-            self.analog_response_table.append(response_entry)
-
-        self.data_lock.release()
+            for pin in range(0, self.number_of_analog_pins_discovered):
+                response_entry = [self.pymata.INPUT, 0]
+                self.analog_response_table.append(response_entry)
 
     #noinspection PyMethodMayBeStatic
     # keeps pycharm happy
@@ -656,11 +628,16 @@ class PyMataCommandHandler(threading.Thread):
         """
         self.analog_mapping_query_results = data
 
+    def stepper_version_response(self, data):
+        """
+        This method handles a stepper library version message sent from the Arduino
+        """
+        self.stepper_library_version = (data[0] & 0x7f) + (data[1] << 7)
+
     def run(self):
         """
         This method starts the thread that continuously runs to receive and interpret
         messages coming from Firmata. This must be the last method in this file
-
         It also checks the deque for messages to be sent to Firmata.
         """
         # To add a command to the command dispatch table, append here.
@@ -675,11 +652,12 @@ class PyMataCommandHandler(threading.Thread):
         self.command_dispatch.update({self.CAPABILITY_RESPONSE: [self.capability_response, 2]})
         self.command_dispatch.update({self.PIN_STATE_RESPONSE: [self.pin_state_response, 2]})
         self.command_dispatch.update({self.ANALOG_MAPPING_RESPONSE: [self.analog_mapping_response, 2]})
+        self.command_dispatch.update({self.STEPPER_DATA: [self.stepper_version_response, 2]})
 
-        while 1:  # a forever loop
-            if len(self.command_deque):
+        while not self.is_stopped():
+            if len(self.pymata.command_deque):
                 # get next byte from the deque and process it
-                data = self.command_deque.popleft()
+                data = self.pymata.command_deque.popleft()
 
                 # this list will be populated with the received data for the command
                 command_data = []
@@ -688,9 +666,9 @@ class PyMataCommandHandler(threading.Thread):
                 if data == self.START_SYSEX:
                     # next char is the actual sysex command
                     # wait until we can get data from the deque
-                    while len(self.command_deque) == 0:
+                    while len(self.pymata.command_deque) == 0:
                         pass
-                    sysex_command = self.command_deque.popleft()
+                    sysex_command = self.pymata.command_deque.popleft()
                     # retrieve the associated command_dispatch entry for this command
                     dispatch_entry = self.command_dispatch.get(sysex_command)
 
@@ -701,9 +679,9 @@ class PyMataCommandHandler(threading.Thread):
                     end_of_sysex = False
                     while not end_of_sysex:
                         # wait for more data to arrive
-                        while len(self.command_deque) == 0:
+                        while len(self.pymata.command_deque) == 0:
                             pass
-                        data = self.command_deque.popleft()
+                        data = self.pymata.command_deque.popleft()
                         if data != self.END_SYSEX:
                             command_data.append(data)
                         else:
@@ -743,9 +721,9 @@ class PyMataCommandHandler(threading.Thread):
                     #look at the number of args that the selected method requires
                     # now get that number of bytes to pass to the called method
                     for i in range(num_args):
-                        while len(self.command_deque) == 0:
+                        while len(self.pymata.command_deque) == 0:
                             pass
-                        data = self.command_deque.popleft()
+                        data = self.pymata.command_deque.popleft()
                         command_data.append(data)
                         #go execute the command with the argument list
                     method(command_data)
